/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMCSSRule.idl"

/**
 * Interface for at-rules that have child rules in the CSS OM.
 */
[scriptable, uuid(ab013eed-fa21-4c6a-bba3-79e8780f583e)]
interface nsIDOMCSSGroupingRule : nsIDOMCSSRule
{
  readonly attribute nsIDOMCSSRuleList cssRules;

  unsigned long      insertRule(in DOMString rule,
                                in unsigned long index)
                                        raises(DOMException);
  void               deleteRule(in unsigned long index)
                                        raises(DOMException);
};
