/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(e65fe6e2-2643-463c-97e2-27665efe2386)]
interface mozIStorageStatementParams : nsISupports {
  // Magic interface for parameter setting that implements nsIXPCScriptable.
};
