/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMXULElement.idl"
interface nsIDOMXULSelectControlElement;

[scriptable, uuid(5cf0f3e0-43aa-4c2b-a599-3adf5025cda5)]
interface nsIDOMXULSelectControlItemElement : nsIDOMXULElement {
  attribute boolean disabled;
  attribute DOMString crop;
  attribute DOMString image;
  attribute DOMString label;
  attribute DOMString accessKey;
  attribute DOMString command;
  
  attribute DOMString value;
  
  readonly attribute boolean selected;
  
  readonly attribute nsIDOMXULSelectControlElement control;
  
  // XXX defined in XULElement, but should be defined here
  // void doCommand();
};


